# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_client_id_facebook import ConnectionClientIdFacebook
from .connection_client_secret_facebook import ConnectionClientSecretFacebook
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_array_facebook import ConnectionScopeArrayFacebook
from .connection_scope_facebook import ConnectionScopeFacebook
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params_facebook import ConnectionUpstreamParamsFacebook


class ConnectionOptionsFacebook(ConnectionOptionsCommon):
    """
    Options for the 'facebook' connection
    """

    client_id: typing.Optional[ConnectionClientIdFacebook] = None
    client_secret: typing.Optional[ConnectionClientSecretFacebook] = None
    freeform_scopes: typing.Optional[ConnectionScopeArrayFacebook] = None
    upstream_params: typing.Optional[ConnectionUpstreamParamsFacebook] = None
    scope: typing.Optional[ConnectionScopeFacebook] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    ads_management: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to both read and manage ads for ad accounts you own or have been granted access to by the owner. By default, your app may only access ad accounts owned by admins of the app when in developer mode.
    """

    ads_read: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants access to the Ads Insights API to pull ads report information for ad accounts you own or have been granted access to by the owner of other ad accounts.
    """

    allow_context_profile_field: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to a social context. Deprecated on April 30th, 2019.
    """

    business_management: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to read and write with the Business Manager API.
    """

    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access a person's primary email address.
    """

    groups_access_member_info: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to publicly available group member information.
    """

    leads_retrieval: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to retrieve all the information captured within a lead.
    """

    manage_notifications: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables your app to read a person's notifications and mark them as read. This permission does not let you send notifications to a person. Deprecated in Graph API v2.3.
    """

    manage_pages: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to retrieve Page Access Tokens for the Pages and Apps that the person administers. Apps need both manage_pages and publish_pages to be able to publish as a Page.
    """

    pages_manage_cta: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Allows the app to perform POST and DELETE operations on endpoints used for managing a Page's Call To Action buttons.
    """

    pages_manage_instant_articles: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to manage Instant Articles on behalf of Facebook Pages administered by people using your app.
    """

    pages_messaging: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to send and receive messages through a Facebook Page.
    """

    pages_messaging_phone_number: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to use the phone number messaging feature.
    """

    pages_messaging_subscriptions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to send messages using Facebook Pages at any time after the first user interaction. Your app may only send advertising or promotional content through sponsored messages or within 24 hours of user interaction.
    """

    pages_show_list: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants access to show the list of the Pages that a person manages.
    """

    public_profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to a user's public profile information including id, first_name, last_name, middle_name, name, name_format, picture, and short_name. This is the most basic permission and is required by Facebook.
    """

    publish_actions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Allows your app to publish to the Open Graph using Built-in Actions, Achievements, Scores, or Custom Actions. Deprecated on August 1st, 2018.
    """

    publish_pages: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to publish posts, comments, and like Pages managed by a person using your app. Your app must also have manage_pages to publish as a Page.
    """

    publish_to_groups: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to post content into a group on behalf of a user who has granted the app this permission.
    """

    publish_video: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to publish live videos to the app User's timeline.
    """

    read_audience_network_insights: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants read-only access to the Audience Network Insights data for Apps the person owns.
    """

    read_insights: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants read-only access to the Insights data for Pages, Apps, and web domains the person owns.
    """

    read_mailbox: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides the ability to read the messages in a person's Facebook Inbox through the inbox edge and the thread node. Deprecated in Graph API v2.3.
    """

    read_page_mailboxes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to read from the Page Inboxes of the Pages managed by a person. This permission is often used alongside the manage_pages permission.
    """

    read_stream: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to read the posts in a person's News Feed, or the posts on their Profile. Deprecated in Graph API v2.3.
    """

    user_age_range: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access a person's age range.
    """

    user_birthday: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access a person's birthday.
    """

    user_events: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants read-only access to the Events a person is a host of or has RSVPed to. This permission is restricted to a limited set of partners and usage requires prior approval by Facebook.
    """

    user_friends: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access a list of friends that also use said app. This permission is restricted to a limited set of partners and usage requires prior approval by Facebook.
    """

    user_gender: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access a person's gender.
    """

    user_groups: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables your app to read the Groups a person is a member of through the groups edge on the User object. Deprecated in Graph API v2.3.
    """

    user_hometown: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access a person's hometown location set in their User Profile.
    """

    user_likes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access the list of all Facebook Pages that a person has liked.
    """

    user_link: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants permission to access the Facebook Profile URL of the user of your app.
    """

    user_location: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to a person's current city through the location field on the User object. The current city is set by a person on their Profile.
    """

    user_managed_groups: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables your app to read the Groups a person is an admin of through the groups edge on the User object. Deprecated in Graph API v3.0.
    """

    user_photos: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to the photos a person has uploaded or been tagged in. This permission is restricted to a limited set of partners and usage requires prior approval by Facebook.
    """

    user_posts: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to the posts on a person's Timeline including their own posts, posts they are tagged in, and posts other people make on their Timeline. This permission is restricted to a limited set of partners and usage requires prior approval by Facebook.
    """

    user_status: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to a person's statuses. These are posts on Facebook which don't include links, videos or photos. Deprecated in Graph API v2.3.
    """

    user_tagged_places: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to the Places a person has been tagged at in photos, videos, statuses and links. This permission is restricted to a limited set of partners and usage requires prior approval by Facebook.
    """

    user_videos: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Provides access to the videos a person has uploaded or been tagged in. This permission is restricted to a limited set of partners and usage requires prior approval by Facebook.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .http_custom_header import HttpCustomHeader
from .log_stream_http_content_format_enum import LogStreamHttpContentFormatEnum


class LogStreamHttpSink(UniversalBaseModel):
    http_authorization: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="httpAuthorization"),
        pydantic.Field(alias="httpAuthorization", description="HTTP Authorization header"),
    ] = None
    """
    HTTP Authorization header
    """

    http_content_format: typing_extensions.Annotated[
        typing.Optional[LogStreamHttpContentFormatEnum],
        FieldMetadata(alias="httpContentFormat"),
        pydantic.Field(alias="httpContentFormat"),
    ] = None
    http_content_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="httpContentType"),
        pydantic.Field(alias="httpContentType", description="HTTP Content-Type header"),
    ] = None
    """
    HTTP Content-Type header
    """

    http_endpoint: typing_extensions.Annotated[
        str, FieldMetadata(alias="httpEndpoint"), pydantic.Field(alias="httpEndpoint", description="HTTP endpoint")
    ]
    """
    HTTP endpoint
    """

    http_custom_headers: typing_extensions.Annotated[
        typing.Optional[typing.List[HttpCustomHeader]],
        FieldMetadata(alias="httpCustomHeaders"),
        pydantic.Field(alias="httpCustomHeaders", description="custom HTTP headers"),
    ] = None
    """
    custom HTTP headers
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_attribute_map_okta import ConnectionAttributeMapOkta
from .connection_domain_okta import ConnectionDomainOkta
from .connection_options_common import ConnectionOptionsCommon
from .connection_options_common_oidc import ConnectionOptionsCommonOidc
from .connection_type_enum_okta import ConnectionTypeEnumOkta


class ConnectionOptionsOkta(ConnectionOptionsCommon, ConnectionOptionsCommonOidc):
    """
    Options for the 'okta' connection
    """

    attribute_map: typing.Optional[ConnectionAttributeMapOkta] = None
    domain: typing.Optional[ConnectionDomainOkta] = None
    type: typing.Optional[ConnectionTypeEnumOkta] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .log_stream_datadog_region_enum import LogStreamDatadogRegionEnum


class LogStreamDatadogSink(UniversalBaseModel):
    datadog_api_key: typing_extensions.Annotated[
        str, FieldMetadata(alias="datadogApiKey"), pydantic.Field(alias="datadogApiKey", description="Datadog API Key")
    ]
    """
    Datadog API Key
    """

    datadog_region: typing_extensions.Annotated[
        LogStreamDatadogRegionEnum, FieldMetadata(alias="datadogRegion"), pydantic.Field(alias="datadogRegion")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ... import core
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.content_too_large_error import ContentTooLargeError
from ...errors.forbidden_error import ForbiddenError
from ...errors.internal_server_error import InternalServerError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.create_import_users_response_content import CreateImportUsersResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawUsersImportsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        users: core.File,
        connection_id: str,
        upsert: typing.Optional[bool] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        send_completion_email: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateImportUsersResponseContent]:
        """
        Import users from a <a href="https://auth0.com/docs/users/references/bulk-import-database-schema-examples">formatted file</a> into a connection via a long-running job. When importing users, with or without upsert, the `email_verified` is set to `false` when the email address is added or updated. Users must verify their email address. To avoid this behavior, set `email_verified` to `true` in the imported data.

        Parameters
        ----------
        users : core.File
            See core.File for more documentation

        connection_id : str
            connection_id of the connection to which users will be imported.

        upsert : typing.Optional[bool]
            Whether to update users if they already exist (true) or to ignore them (false).

        external_id : typing.Optional[str]
            Customer-defined ID.

        send_completion_email : typing.Optional[bool]
            Whether to send a completion email to all tenant owners when the job is finished (true) or not (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateImportUsersResponseContent]
            Job successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "jobs/users-imports",
            method="POST",
            data={
                "connection_id": connection_id,
                "upsert": upsert,
                "external_id": external_id,
                "send_completion_email": send_completion_email,
            },
            files={
                "users": users,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateImportUsersResponseContent,
                    parse_obj_as(
                        type_=CreateImportUsersResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 413:
                raise ContentTooLargeError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawUsersImportsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        users: core.File,
        connection_id: str,
        upsert: typing.Optional[bool] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        send_completion_email: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateImportUsersResponseContent]:
        """
        Import users from a <a href="https://auth0.com/docs/users/references/bulk-import-database-schema-examples">formatted file</a> into a connection via a long-running job. When importing users, with or without upsert, the `email_verified` is set to `false` when the email address is added or updated. Users must verify their email address. To avoid this behavior, set `email_verified` to `true` in the imported data.

        Parameters
        ----------
        users : core.File
            See core.File for more documentation

        connection_id : str
            connection_id of the connection to which users will be imported.

        upsert : typing.Optional[bool]
            Whether to update users if they already exist (true) or to ignore them (false).

        external_id : typing.Optional[str]
            Customer-defined ID.

        send_completion_email : typing.Optional[bool]
            Whether to send a completion email to all tenant owners when the job is finished (true) or not (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateImportUsersResponseContent]
            Job successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "jobs/users-imports",
            method="POST",
            data={
                "connection_id": connection_id,
                "upsert": upsert,
                "external_id": external_id,
                "send_completion_email": send_completion_email,
            },
            files={
                "users": users,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateImportUsersResponseContent,
                    parse_obj_as(
                        type_=CreateImportUsersResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 413:
                raise ContentTooLargeError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .email_smtp_host import EmailSmtpHost


class EmailProviderCredentialsSchemaSmtpHost(UniversalBaseModel):
    smtp_host: typing.Optional[EmailSmtpHost] = None
    smtp_port: typing.Optional[int] = pydantic.Field(default=None)
    """
    SMTP port.
    """

    smtp_user: typing.Optional[str] = pydantic.Field(default=None)
    """
    SMTP username.
    """

    smtp_pass: typing.Optional[str] = pydantic.Field(default=None)
    """
    SMTP password.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_json_serialize_json_params_object_object import FlowActionJsonSerializeJsonParamsObjectObject

FlowActionJsonSerializeJsonParamsObject = typing.Union[str, FlowActionJsonSerializeJsonParamsObjectObject]



# This file was auto-generated by Fern from our API Definition.

import typing

from .form_field_payment_config_charge_one_off import FormFieldPaymentConfigChargeOneOff
from .form_field_payment_config_charge_subscription import FormFieldPaymentConfigChargeSubscription

FormFieldPaymentConfigCharge = typing.Union[
    FormFieldPaymentConfigChargeOneOff, FormFieldPaymentConfigChargeSubscription
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .connection_app_domain_azure_ad import ConnectionAppDomainAzureAd
from .connection_client_id_azure_ad import ConnectionClientIdAzureAd
from .connection_client_secret_azure_ad import ConnectionClientSecretAzureAd
from .connection_domain_aliases_azure_ad import ConnectionDomainAliasesAzureAd
from .connection_ext_admin import ConnectionExtAdmin
from .connection_ext_agreed_terms import ConnectionExtAgreedTerms
from .connection_ext_assigned_plans import ConnectionExtAssignedPlans
from .connection_ext_groups_azure_ad import ConnectionExtGroupsAzureAd
from .connection_ext_is_suspended import ConnectionExtIsSuspended
from .connection_ext_profile import ConnectionExtProfile
from .connection_icon_url_azure_ad import ConnectionIconUrlAzureAd
from .connection_identity_api_azure_ad import ConnectionIdentityApiAzureAd
from .connection_max_groups_to_retrieve import ConnectionMaxGroupsToRetrieve
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_azure_ad import ConnectionScopeAzureAd
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_should_trust_email_verified_connection_enum import ConnectionShouldTrustEmailVerifiedConnectionEnum
from .connection_tenant_domain_azure_ad_one import ConnectionTenantDomainAzureAdOne
from .connection_tenant_id_azure_ad import ConnectionTenantIdAzureAd
from .connection_thumbprints import ConnectionThumbprints
from .connection_upstream_params import ConnectionUpstreamParams
from .connection_use_common_endpoint_azure_ad import ConnectionUseCommonEndpointAzureAd
from .connection_userid_attribute_azure_ad import ConnectionUseridAttributeAzureAd
from .connection_waad_protocol import ConnectionWaadProtocol


class ConnectionOptionsAzureAd(ConnectionOptionsCommon):
    """
    Options for the 'waad' connection
    """

    api_enable_users: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable users API
    """

    app_domain: typing.Optional[ConnectionAppDomainAzureAd] = None
    app_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Application ID URI (App ID URI) for the Azure AD application. Required when using Azure AD v1 with the Resource Owner Password flow. Used to identify the resource being requested in OAuth token requests.
    """

    basic_profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Includes basic user profile information from Azure AD (name, email, given_name, family_name). Always enabled and required - represents the minimum profile data retrieved during authentication.
    """

    client_id: ConnectionClientIdAzureAd
    client_secret: typing.Optional[ConnectionClientSecretAzureAd] = None
    domain_aliases: typing.Optional[ConnectionDomainAliasesAzureAd] = None
    ext_access_token: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's Azure AD access token in the Auth0 user profile. When true (default), the access token is persisted for API access.
    """

    ext_account_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing whether the user's Azure AD account is enabled. When true (default), the account enabled status is persisted in the user profile.
    """

    ext_admin: typing.Optional[ConnectionExtAdmin] = None
    ext_agreed_terms: typing.Optional[ConnectionExtAgreedTerms] = None
    ext_assigned_licenses: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the list of Microsoft 365/Office 365 licenses assigned to the user. When true (default), license information is persisted in the user profile.
    """

    ext_assigned_plans: typing.Optional[ConnectionExtAssignedPlans] = None
    ext_azure_id: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's Azure ID identifier. When true (default), the Azure ID is persisted. Note: 'oid' (Object ID) is the recommended unique identifier for single-tenant connections.
    """

    ext_city: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's city from Azure AD. When true (default), city information is persisted in the user profile.
    """

    ext_country: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's country from Azure AD. When true (default), country information is persisted in the user profile.
    """

    ext_department: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's department from Azure AD. When true (default), department information is persisted in the user profile.
    """

    ext_dir_sync_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing whether directory synchronization is enabled for the user. When true (default), directory sync status is persisted in the user profile.
    """

    ext_email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's email address from Azure AD. When true (default), email is persisted in the user profile.
    """

    ext_expires_in: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the token expiration time (in seconds). When true (default), expiration information is persisted in the user profile.
    """

    ext_family_name: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's family name (last name) from Azure AD. When true (default), family name is persisted in the user profile.
    """

    ext_fax: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's fax number from Azure AD. When true (default), fax information is persisted in the user profile.
    """

    ext_given_name: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's given name (first name) from Azure AD. When true (default), given name is persisted in the user profile.
    """

    ext_group_ids: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the list of Azure AD group IDs the user is a member of. When true (default), group membership IDs are persisted. See ext_groups for retrieving group details.
    """

    ext_groups: typing.Optional[ConnectionExtGroupsAzureAd] = None
    ext_is_suspended: typing.Optional[ConnectionExtIsSuspended] = None
    ext_job_title: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's job title from Azure AD. When true (default), job title information is persisted in the user profile.
    """

    ext_last_sync: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the timestamp of the last directory synchronization. When true (default), the last sync date is persisted in the user profile.
    """

    ext_mobile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's mobile phone number from Azure AD. When true (default), mobile number is persisted in the user profile.
    """

    ext_name: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's full name from Azure AD. When true (default), full name is persisted in the user profile.
    """

    ext_nested_groups: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, stores all groups the user is member of, including transitive group memberships (groups within groups). When false (default), only direct group memberships are included.
    """

    ext_nickname: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's nickname or display name from Azure AD. When true (default), nickname is persisted in the user profile.
    """

    ext_oid: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's Object ID (oid) from Azure AD. When true (default), the oid is persisted. Note: 'oid' is the recommended unique identifier for single-tenant connections and required for SCIM.
    """

    ext_phone: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's phone number from Azure AD. When true (default), phone number is persisted in the user profile.
    """

    ext_physical_delivery_office_name: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's office location from Azure AD. When true (default), office location is persisted in the user profile.
    """

    ext_postal_code: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's postal code from Azure AD. When true (default), postal code is persisted in the user profile.
    """

    ext_preferred_language: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's preferred language from Azure AD. When true (default), language preference is persisted in the user profile.
    """

    ext_profile: typing.Optional[ConnectionExtProfile] = None
    ext_provisioned_plans: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the list of service plans provisioned to the user. When true (default), provisioned plans are persisted in the user profile.
    """

    ext_provisioning_errors: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing provisioning errors that occurred during synchronization. When true (default), error information is persisted. Useful for troubleshooting sync issues.
    """

    ext_proxy_addresses: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing all proxy email addresses (email aliases) for the user. When true (default), proxy addresses are persisted in the user profile.
    """

    ext_puid: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's Passport User ID (puid). When true (default), puid is persisted in the user profile. Legacy attribute.
    """

    ext_refresh_token: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the Azure AD refresh token. When true (default), the refresh token is persisted for offline access. Required for token refresh in long-lived applications.
    """

    ext_roles: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing Azure AD application roles assigned to the user. When true (default), role information is persisted. Useful for RBAC in applications.
    """

    ext_state: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's state (province/region) from Azure AD. When true (default), state information is persisted in the user profile.
    """

    ext_street: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's street address from Azure AD. When true (default), street address is persisted in the user profile.
    """

    ext_telephone_number: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="ext_telephoneNumber"),
        pydantic.Field(
            alias="ext_telephoneNumber",
            description="When false, prevents storing the user's telephone number from Azure AD. When true (default), telephone number is persisted in the user profile.",
        ),
    ] = None
    """
    When false, prevents storing the user's telephone number from Azure AD. When true (default), telephone number is persisted in the user profile.
    """

    ext_tenantid: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's Azure AD tenant ID. When true (default), tenant ID is persisted. Useful for identifying which Azure AD organization the user belongs to.
    """

    ext_upn: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's User Principal Name (UPN) from Azure AD. When true (default), UPN is persisted. UPN is the user's logon name (e.g., user@contoso.com).
    """

    ext_usage_location: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing the user's usage location for license assignment. When true (default), usage location is persisted in the user profile.
    """

    ext_user_id: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When false, prevents storing an alternative user ID. When true (default), this user ID is persisted in the user profile.
    """

    granted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether admin consent has been granted for the required Azure AD permissions. Read-only status field managed by Auth0 during the OAuth authorization flow.
    """

    icon_url: typing.Optional[ConnectionIconUrlAzureAd] = None
    identity_api: typing.Optional[ConnectionIdentityApiAzureAd] = None
    max_groups_to_retrieve: typing.Optional[ConnectionMaxGroupsToRetrieve] = None
    scope: typing.Optional[ConnectionScopeAzureAd] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    should_trust_email_verified_connection: typing.Optional[ConnectionShouldTrustEmailVerifiedConnectionEnum] = None
    tenant_domain: typing.Optional[ConnectionTenantDomainAzureAdOne] = None
    tenant_id: typing_extensions.Annotated[
        typing.Optional[ConnectionTenantIdAzureAd], FieldMetadata(alias="tenantId"), pydantic.Field(alias="tenantId")
    ] = None
    thumbprints: typing.Optional[ConnectionThumbprints] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    use_wsfed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates WS-Federation protocol usage. When true, uses WS-Federation; when false, uses OpenID Connect.
    """

    use_common_endpoint: typing_extensions.Annotated[
        typing.Optional[ConnectionUseCommonEndpointAzureAd],
        FieldMetadata(alias="useCommonEndpoint"),
        pydantic.Field(alias="useCommonEndpoint"),
    ] = None
    userid_attribute: typing.Optional[ConnectionUseridAttributeAzureAd] = None
    waad_protocol: typing.Optional[ConnectionWaadProtocol] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FlowActionWhatsappSendMessageParamsType = typing.Union[
    typing.Literal["AUDIO", "CONTACTS", "DOCUMENT", "IMAGE", "INTERACTIVE", "LOCATION", "STICKER", "TEMPLATE", "TEXT"],
    typing.Any,
]



from .conftest import get_client, verify_request_count


def test_guardian_factors_duo_settings_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "guardian.factors.duo.settings.get.0"
    client = get_client(test_id)
    client.guardian.factors.duo.settings.get()
    verify_request_count(test_id, "GET", "/guardian/factors/duo/settings", None, 1)


def test_guardian_factors_duo_settings_set_() -> None:
    """Test set endpoint with WireMock"""
    test_id = "guardian.factors.duo.settings.set_.0"
    client = get_client(test_id)
    client.guardian.factors.duo.settings.set()
    verify_request_count(test_id, "PUT", "/guardian/factors/duo/settings", None, 1)


def test_guardian_factors_duo_settings_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "guardian.factors.duo.settings.update.0"
    client = get_client(test_id)
    client.guardian.factors.duo.settings.update()
    verify_request_count(test_id, "PATCH", "/guardian/factors/duo/settings", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .flow_action_google_sheets_add_row_action import FlowActionGoogleSheetsAddRowAction
from .flow_action_google_sheets_add_row_params import FlowActionGoogleSheetsAddRowParams
from .flow_action_google_sheets_add_row_type import FlowActionGoogleSheetsAddRowType


class FlowActionGoogleSheetsAddRow(UniversalBaseModel):
    id: str
    alias: typing.Optional[str] = None
    type: FlowActionGoogleSheetsAddRowType
    action: FlowActionGoogleSheetsAddRowAction
    allow_failure: typing.Optional[bool] = None
    mask_output: typing.Optional[bool] = None
    params: FlowActionGoogleSheetsAddRowParams

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_create_authentication_method_private_key_jwt import ClientCreateAuthenticationMethodPrivateKeyJwt
from .client_create_authentication_method_tls_client_auth import ClientCreateAuthenticationMethodTlsClientAuth
from .create_client_authentication_method_self_signed_tls_client_auth import (
    CreateClientAuthenticationMethodSelfSignedTlsClientAuth,
)


class ClientCreateAuthenticationMethod(UniversalBaseModel):
    """
    Defines client authentication methods.
    """

    private_key_jwt: typing.Optional[ClientCreateAuthenticationMethodPrivateKeyJwt] = None
    tls_client_auth: typing.Optional[ClientCreateAuthenticationMethodTlsClientAuth] = None
    self_signed_tls_client_auth: typing.Optional[CreateClientAuthenticationMethodSelfSignedTlsClientAuth] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ClientOidcBackchannelLogoutInitiatorsEnum = typing.Union[
    typing.Literal[
        "rp-logout",
        "idp-logout",
        "password-changed",
        "session-expired",
        "session-revoked",
        "account-deleted",
        "email-identifier-changed",
        "mfa-phone-unenrolled",
        "account-deactivated",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_zapier_uninitialized import CreateFlowsVaultConnectionZapierUninitialized
from .create_flows_vault_connection_zapier_webhook import CreateFlowsVaultConnectionZapierWebhook

CreateFlowsVaultConnectionZapier = typing.Union[
    CreateFlowsVaultConnectionZapierWebhook, CreateFlowsVaultConnectionZapierUninitialized
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_build_status_enum import ActionBuildStatusEnum
from .action_deployed_version import ActionDeployedVersion
from .action_module_reference import ActionModuleReference
from .action_secret_response import ActionSecretResponse
from .action_trigger import ActionTrigger
from .action_version_dependency import ActionVersionDependency
from .integration import Integration


class GetActionResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID of the action.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of an action.
    """

    supported_triggers: typing.Optional[typing.List[ActionTrigger]] = pydantic.Field(default=None)
    """
    The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.
    """

    all_changes_deployed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if all of an Action's contents have been deployed.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this action was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this action was updated.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The source code of the action.
    """

    dependencies: typing.Optional[typing.List[ActionVersionDependency]] = pydantic.Field(default=None)
    """
    The list of third party npm modules, and their versions, that this action depends on.
    """

    runtime: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Node runtime. For example: `node22`, defaults to `node22`
    """

    secrets: typing.Optional[typing.List[ActionSecretResponse]] = pydantic.Field(default=None)
    """
    The list of secrets that are included in an action or a version of an action.
    """

    deployed_version: typing.Optional[ActionDeployedVersion] = None
    installed_integration_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    installed_integration_id is the fk reference to the InstalledIntegration entity.
    """

    integration: typing.Optional[Integration] = None
    status: typing.Optional[ActionBuildStatusEnum] = None
    built_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this action was built successfully.
    """

    deploy: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the action should be deployed after creation.
    """

    modules: typing.Optional[typing.List[ActionModuleReference]] = pydantic.Field(default=None)
    """
    The list of action modules and their versions used by this action.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.acul_configs import AculConfigs
from ...types.acul_context_configuration import AculContextConfiguration
from ...types.acul_filters import AculFilters
from ...types.acul_head_tag import AculHeadTag
from ...types.acul_rendering_mode_enum import AculRenderingModeEnum
from ...types.bulk_update_acul_response_content import BulkUpdateAculResponseContent
from ...types.get_acul_response_content import GetAculResponseContent
from ...types.list_aculs_offset_paginated_response_content import ListAculsOffsetPaginatedResponseContent
from ...types.list_aculs_response_content_item import ListAculsResponseContentItem
from ...types.prompt_group_name_enum import PromptGroupNameEnum
from ...types.screen_group_name_enum import ScreenGroupNameEnum
from ...types.update_acul_response_content import UpdateAculResponseContent
from .raw_client import AsyncRawRenderingClient, RawRenderingClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RenderingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRenderingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRenderingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRenderingClient
        """
        return self._raw_client

    def list(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        prompt: typing.Optional[str] = None,
        screen: typing.Optional[str] = None,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]:
        """
        Get render setting configurations for all screens.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (default: true) or excluded (false).

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Maximum value is 100, default value is 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total configuration count (true) or as a direct array of results (false, default).

        prompt : typing.Optional[str]
            Name of the prompt to filter by

        screen : typing.Optional[str]
            Name of the screen to filter by

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode to filter by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]
            ACUL settings successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.prompts.rendering.list(
            fields="fields",
            include_fields=True,
            page=1,
            per_page=1,
            include_totals=True,
            prompt="prompt",
            screen="screen",
            rendering_mode="advanced",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            fields=fields,
            include_fields=include_fields,
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            prompt=prompt,
            screen=screen,
            rendering_mode=rendering_mode,
            request_options=request_options,
        )

    def bulk_update(
        self, *, configs: AculConfigs, request_options: typing.Optional[RequestOptions] = None
    ) -> BulkUpdateAculResponseContent:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        configs : AculConfigs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpdateAculResponseContent
            ACUL settings successfully updated.

        Examples
        --------
        from auth0 import AculConfigsItem, Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.rendering.bulk_update(
            configs=[
                AculConfigsItem(
                    prompt="login",
                    screen="login",
                )
            ],
        )
        """
        _response = self._raw_client.bulk_update(configs=configs, request_options=request_options)
        return _response.data

    def get(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAculResponseContent:
        """
        Get render settings for a screen.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAculResponseContent
            ACUL settings successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.rendering.get(
            prompt="login",
            screen="login",
        )
        """
        _response = self._raw_client.get(prompt, screen, request_options=request_options)
        return _response.data

    def update(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = OMIT,
        context_configuration: typing.Optional[AculContextConfiguration] = OMIT,
        default_head_tags_disabled: typing.Optional[bool] = OMIT,
        use_page_template: typing.Optional[bool] = OMIT,
        head_tags: typing.Optional[typing.Sequence[AculHeadTag]] = OMIT,
        filters: typing.Optional[AculFilters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateAculResponseContent:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode

        context_configuration : typing.Optional[AculContextConfiguration]

        default_head_tags_disabled : typing.Optional[bool]
            Override Universal Login default head tags

        use_page_template : typing.Optional[bool]
            Use page template with ACUL

        head_tags : typing.Optional[typing.Sequence[AculHeadTag]]
            An array of head tags

        filters : typing.Optional[AculFilters]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateAculResponseContent
            ACUL settings successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.rendering.update(
            prompt="login",
            screen="login",
        )
        """
        _response = self._raw_client.update(
            prompt,
            screen,
            rendering_mode=rendering_mode,
            context_configuration=context_configuration,
            default_head_tags_disabled=default_head_tags_disabled,
            use_page_template=use_page_template,
            head_tags=head_tags,
            filters=filters,
            request_options=request_options,
        )
        return _response.data


class AsyncRenderingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRenderingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRenderingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRenderingClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        prompt: typing.Optional[str] = None,
        screen: typing.Optional[str] = None,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]:
        """
        Get render setting configurations for all screens.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (default: true) or excluded (false).

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Maximum value is 100, default value is 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total configuration count (true) or as a direct array of results (false, default).

        prompt : typing.Optional[str]
            Name of the prompt to filter by

        screen : typing.Optional[str]
            Name of the screen to filter by

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode to filter by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]
            ACUL settings successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.prompts.rendering.list(
                fields="fields",
                include_fields=True,
                page=1,
                per_page=1,
                include_totals=True,
                prompt="prompt",
                screen="screen",
                rendering_mode="advanced",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            fields=fields,
            include_fields=include_fields,
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            prompt=prompt,
            screen=screen,
            rendering_mode=rendering_mode,
            request_options=request_options,
        )

    async def bulk_update(
        self, *, configs: AculConfigs, request_options: typing.Optional[RequestOptions] = None
    ) -> BulkUpdateAculResponseContent:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        configs : AculConfigs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpdateAculResponseContent
            ACUL settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AculConfigsItem, AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.rendering.bulk_update(
                configs=[
                    AculConfigsItem(
                        prompt="login",
                        screen="login",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_update(configs=configs, request_options=request_options)
        return _response.data

    async def get(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAculResponseContent:
        """
        Get render settings for a screen.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAculResponseContent
            ACUL settings successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.rendering.get(
                prompt="login",
                screen="login",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(prompt, screen, request_options=request_options)
        return _response.data

    async def update(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = OMIT,
        context_configuration: typing.Optional[AculContextConfiguration] = OMIT,
        default_head_tags_disabled: typing.Optional[bool] = OMIT,
        use_page_template: typing.Optional[bool] = OMIT,
        head_tags: typing.Optional[typing.Sequence[AculHeadTag]] = OMIT,
        filters: typing.Optional[AculFilters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateAculResponseContent:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode

        context_configuration : typing.Optional[AculContextConfiguration]

        default_head_tags_disabled : typing.Optional[bool]
            Override Universal Login default head tags

        use_page_template : typing.Optional[bool]
            Use page template with ACUL

        head_tags : typing.Optional[typing.Sequence[AculHeadTag]]
            An array of head tags

        filters : typing.Optional[AculFilters]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateAculResponseContent
            ACUL settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.rendering.update(
                prompt="login",
                screen="login",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            prompt,
            screen,
            rendering_mode=rendering_mode,
            context_configuration=context_configuration,
            default_head_tags_disabled=default_head_tags_disabled,
            use_page_template=use_page_template,
            head_tags=head_tags,
            filters=filters,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.connection_name_prefix_template import ConnectionNamePrefixTemplate
from ..types.connection_profile import ConnectionProfile
from ..types.connection_profile_config import ConnectionProfileConfig
from ..types.connection_profile_enabled_features import ConnectionProfileEnabledFeatures
from ..types.connection_profile_name import ConnectionProfileName
from ..types.connection_profile_organization import ConnectionProfileOrganization
from ..types.connection_profile_strategy_overrides import ConnectionProfileStrategyOverrides
from ..types.create_connection_profile_response_content import CreateConnectionProfileResponseContent
from ..types.get_connection_profile_response_content import GetConnectionProfileResponseContent
from ..types.get_connection_profile_template_response_content import GetConnectionProfileTemplateResponseContent
from ..types.list_connection_profile_template_response_content import ListConnectionProfileTemplateResponseContent
from ..types.list_connection_profiles_paginated_response_content import ListConnectionProfilesPaginatedResponseContent
from ..types.update_connection_profile_response_content import UpdateConnectionProfileResponseContent
from .raw_client import AsyncRawConnectionProfilesClient, RawConnectionProfilesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConnectionProfilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConnectionProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConnectionProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConnectionProfilesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]:
        """
        Retrieve a list of Connection Profiles. This endpoint supports Checkpoint pagination.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 5.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]
            Connection Profiles successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.connection_profiles.list(
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(from_=from_, take=take, request_options=request_options)

    def create(
        self,
        *,
        name: ConnectionProfileName,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateConnectionProfileResponseContent:
        """
        Create a Connection Profile.

        Parameters
        ----------
        name : ConnectionProfileName

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateConnectionProfileResponseContent
            Connection profile successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connection_profiles.create(
            name="name",
        )
        """
        _response = self._raw_client.create(
            name=name,
            organization=organization,
            connection_name_prefix_template=connection_name_prefix_template,
            enabled_features=enabled_features,
            connection_config=connection_config,
            strategy_overrides=strategy_overrides,
            request_options=request_options,
        )
        return _response.data

    def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ListConnectionProfileTemplateResponseContent:
        """
        Retrieve a list of Connection Profile Templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListConnectionProfileTemplateResponseContent
            Connection Profile Templates successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connection_profiles.list_templates()
        """
        _response = self._raw_client.list_templates(request_options=request_options)
        return _response.data

    def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetConnectionProfileTemplateResponseContent:
        """
        Retrieve a Connection Profile Template.

        Parameters
        ----------
        id : str
            ID of the connection-profile-template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConnectionProfileTemplateResponseContent
            Connection Profile Template successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connection_profiles.get_template(
            id="id",
        )
        """
        _response = self._raw_client.get_template(id, request_options=request_options)
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetConnectionProfileResponseContent:
        """
        Retrieve details about a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConnectionProfileResponseContent
            Record for existing connection profile.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connection_profiles.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connection_profiles.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[ConnectionProfileName] = OMIT,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateConnectionProfileResponseContent:
        """
        Update the details of a specific Connection Profile.

        Parameters
        ----------
        id : str
            ID of the connection profile to update.

        name : typing.Optional[ConnectionProfileName]

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateConnectionProfileResponseContent
            Connection profile successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connection_profiles.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            name=name,
            organization=organization,
            connection_name_prefix_template=connection_name_prefix_template,
            enabled_features=enabled_features,
            connection_config=connection_config,
            strategy_overrides=strategy_overrides,
            request_options=request_options,
        )
        return _response.data


class AsyncConnectionProfilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConnectionProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConnectionProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConnectionProfilesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]:
        """
        Retrieve a list of Connection Profiles. This endpoint supports Checkpoint pagination.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 5.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]
            Connection Profiles successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.connection_profiles.list(
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(from_=from_, take=take, request_options=request_options)

    async def create(
        self,
        *,
        name: ConnectionProfileName,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateConnectionProfileResponseContent:
        """
        Create a Connection Profile.

        Parameters
        ----------
        name : ConnectionProfileName

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateConnectionProfileResponseContent
            Connection profile successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connection_profiles.create(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            organization=organization,
            connection_name_prefix_template=connection_name_prefix_template,
            enabled_features=enabled_features,
            connection_config=connection_config,
            strategy_overrides=strategy_overrides,
            request_options=request_options,
        )
        return _response.data

    async def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ListConnectionProfileTemplateResponseContent:
        """
        Retrieve a list of Connection Profile Templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListConnectionProfileTemplateResponseContent
            Connection Profile Templates successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connection_profiles.list_templates()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_templates(request_options=request_options)
        return _response.data

    async def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetConnectionProfileTemplateResponseContent:
        """
        Retrieve a Connection Profile Template.

        Parameters
        ----------
        id : str
            ID of the connection-profile-template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConnectionProfileTemplateResponseContent
            Connection Profile Template successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connection_profiles.get_template(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_template(id, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetConnectionProfileResponseContent:
        """
        Retrieve details about a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConnectionProfileResponseContent
            Record for existing connection profile.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connection_profiles.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connection_profiles.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[ConnectionProfileName] = OMIT,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateConnectionProfileResponseContent:
        """
        Update the details of a specific Connection Profile.

        Parameters
        ----------
        id : str
            ID of the connection profile to update.

        name : typing.Optional[ConnectionProfileName]

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateConnectionProfileResponseContent
            Connection profile successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connection_profiles.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            name=name,
            organization=organization,
            connection_name_prefix_template=connection_name_prefix_template,
            enabled_features=enabled_features,
            connection_config=connection_config,
            strategy_overrides=strategy_overrides,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

UserEnrollmentAuthMethodEnum = typing.Union[
    typing.Literal["authenticator", "guardian", "sms", "webauthn-platform", "webauthn-roaming"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

CustomDomainProvisioningTypeEnum = typing.Union[typing.Literal["auth0_managed_certs", "self_managed_certs"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_oidc_backchannel_logout_initiators import ClientOidcBackchannelLogoutInitiators
from .client_oidc_backchannel_logout_session_metadata import ClientOidcBackchannelLogoutSessionMetadata


class ClientOidcBackchannelLogoutSettings(UniversalBaseModel):
    """
    Configuration for OIDC backchannel logout
    """

    backchannel_logout_urls: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Comma-separated list of URLs that are valid to call back from Auth0 for OIDC backchannel logout. Currently only one URL is allowed.
    """

    backchannel_logout_initiators: typing.Optional[ClientOidcBackchannelLogoutInitiators] = None
    backchannel_logout_session_metadata: typing.Optional[ClientOidcBackchannelLogoutSessionMetadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_connection_updated_object_0 import EventStreamCloudEventConnectionUpdatedObject0
from .event_stream_cloud_event_connection_updated_object_1 import EventStreamCloudEventConnectionUpdatedObject1
from .event_stream_cloud_event_connection_updated_object_2 import EventStreamCloudEventConnectionUpdatedObject2
from .event_stream_cloud_event_connection_updated_object_3 import EventStreamCloudEventConnectionUpdatedObject3
from .event_stream_cloud_event_connection_updated_object_4 import EventStreamCloudEventConnectionUpdatedObject4
from .event_stream_cloud_event_connection_updated_object_5 import EventStreamCloudEventConnectionUpdatedObject5
from .event_stream_cloud_event_connection_updated_object_6 import EventStreamCloudEventConnectionUpdatedObject6
from .event_stream_cloud_event_connection_updated_object_7 import EventStreamCloudEventConnectionUpdatedObject7

EventStreamCloudEventConnectionUpdatedObject = typing.Union[
    EventStreamCloudEventConnectionUpdatedObject0,
    EventStreamCloudEventConnectionUpdatedObject1,
    EventStreamCloudEventConnectionUpdatedObject2,
    EventStreamCloudEventConnectionUpdatedObject3,
    EventStreamCloudEventConnectionUpdatedObject4,
    EventStreamCloudEventConnectionUpdatedObject5,
    EventStreamCloudEventConnectionUpdatedObject6,
    EventStreamCloudEventConnectionUpdatedObject7,
]



# This file was auto-generated by Fern from our API Definition.

from .connection_api_enable_users import ConnectionApiEnableUsers

ConnectionApiEnableUsersGoogleApps = ConnectionApiEnableUsers
"""
Enables integration with the Google Workspace Admin SDK Directory API. When true, Auth0 can retrieve extended user attributes (admin status, suspension status, group memberships) and supports inbound directory provisioning (SCIM). Defaults to true. 
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionGatewayAuthenticationMethodSms = str
"""
The Authorization header type when calling the SMS gateway. Set to 'bearer' for JWT token authentication.
"""



from .conftest import get_client, verify_request_count

from auth0.management import CreateLogStreamHttpRequestBody, LogStreamHttpSink


def test_logStreams_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "log_streams.list_.0"
    client = get_client(test_id)
    client.log_streams.list()
    verify_request_count(test_id, "GET", "/log-streams", None, 1)


def test_logStreams_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "log_streams.create.0"
    client = get_client(test_id)
    client.log_streams.create(
        request=CreateLogStreamHttpRequestBody(
            type="http",
            sink=LogStreamHttpSink(
                http_endpoint="httpEndpoint",
            ),
        ),
    )
    verify_request_count(test_id, "POST", "/log-streams", None, 1)


def test_logStreams_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "log_streams.get.0"
    client = get_client(test_id)
    client.log_streams.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/log-streams/id", None, 1)


def test_logStreams_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "log_streams.delete.0"
    client = get_client(test_id)
    client.log_streams.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/log-streams/id", None, 1)


def test_logStreams_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "log_streams.update.0"
    client = get_client(test_id)
    client.log_streams.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/log-streams/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .rate_limit_policy_configuration import RateLimitPolicyConfiguration
from .rate_limit_policy_consumer_enum import RateLimitPolicyConsumerEnum
from .rate_limit_policy_resource_enum import RateLimitPolicyResourceEnum


class UpdateRateLimitPolicyResponseContent(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    Unique identifier for the Rate Limit Policy.
    """

    resource: RateLimitPolicyResourceEnum
    consumer: RateLimitPolicyConsumerEnum
    consumer_selector: str = pydantic.Field()
    """
    Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.
    """

    configuration: RateLimitPolicyConfiguration
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date and time when the rate limit policy was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date and time when the rate limit policy was last updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count

from auth0.management import PostConnectionKeysRequestContent, RotateConnectionKeysRequestContent


def test_connections_keys_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "connections.keys.get.0"
    client = get_client(test_id)
    client.connections.keys.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/connections/id/keys", None, 1)


def test_connections_keys_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "connections.keys.create.0"
    client = get_client(test_id)
    client.connections.keys.create(
        id="id",
        request=PostConnectionKeysRequestContent(),
    )
    verify_request_count(test_id, "POST", "/connections/id/keys", None, 1)


def test_connections_keys_rotate() -> None:
    """Test rotate endpoint with WireMock"""
    test_id = "connections.keys.rotate.0"
    client = get_client(test_id)
    client.connections.keys.rotate(
        id="id",
        request=RotateConnectionKeysRequestContent(),
    )
    verify_request_count(test_id, "POST", "/connections/id/keys/rotate", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.attack_protection_captcha_auth_challenge_request import AttackProtectionCaptchaAuthChallengeRequest
from ...types.attack_protection_captcha_provider_id import AttackProtectionCaptchaProviderId
from ...types.attack_protection_captcha_simple_captcha_response_content import (
    AttackProtectionCaptchaSimpleCaptchaResponseContent,
)
from ...types.attack_protection_update_captcha_arkose import AttackProtectionUpdateCaptchaArkose
from ...types.attack_protection_update_captcha_friendly_captcha import AttackProtectionUpdateCaptchaFriendlyCaptcha
from ...types.attack_protection_update_captcha_hcaptcha import AttackProtectionUpdateCaptchaHcaptcha
from ...types.attack_protection_update_captcha_recaptcha_enterprise import (
    AttackProtectionUpdateCaptchaRecaptchaEnterprise,
)
from ...types.attack_protection_update_captcha_recaptcha_v_2 import AttackProtectionUpdateCaptchaRecaptchaV2
from ...types.get_attack_protection_captcha_response_content import GetAttackProtectionCaptchaResponseContent
from ...types.update_attack_protection_captcha_response_content import UpdateAttackProtectionCaptchaResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCaptchaClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetAttackProtectionCaptchaResponseContent]:
        """
        Get the CAPTCHA configuration for your client.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetAttackProtectionCaptchaResponseContent]
            Captcha configuration successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/captcha",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAttackProtectionCaptchaResponseContent,
                    parse_obj_as(
                        type_=GetAttackProtectionCaptchaResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        active_provider_id: typing.Optional[AttackProtectionCaptchaProviderId] = OMIT,
        arkose: typing.Optional[AttackProtectionUpdateCaptchaArkose] = OMIT,
        auth_challenge: typing.Optional[AttackProtectionCaptchaAuthChallengeRequest] = OMIT,
        hcaptcha: typing.Optional[AttackProtectionUpdateCaptchaHcaptcha] = OMIT,
        friendly_captcha: typing.Optional[AttackProtectionUpdateCaptchaFriendlyCaptcha] = OMIT,
        recaptcha_enterprise: typing.Optional[AttackProtectionUpdateCaptchaRecaptchaEnterprise] = OMIT,
        recaptcha_v_2: typing.Optional[AttackProtectionUpdateCaptchaRecaptchaV2] = OMIT,
        simple_captcha: typing.Optional[AttackProtectionCaptchaSimpleCaptchaResponseContent] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateAttackProtectionCaptchaResponseContent]:
        """
        Update existing CAPTCHA configuration for your client.

        Parameters
        ----------
        active_provider_id : typing.Optional[AttackProtectionCaptchaProviderId]

        arkose : typing.Optional[AttackProtectionUpdateCaptchaArkose]

        auth_challenge : typing.Optional[AttackProtectionCaptchaAuthChallengeRequest]

        hcaptcha : typing.Optional[AttackProtectionUpdateCaptchaHcaptcha]

        friendly_captcha : typing.Optional[AttackProtectionUpdateCaptchaFriendlyCaptcha]

        recaptcha_enterprise : typing.Optional[AttackProtectionUpdateCaptchaRecaptchaEnterprise]

        recaptcha_v_2 : typing.Optional[AttackProtectionUpdateCaptchaRecaptchaV2]

        simple_captcha : typing.Optional[AttackProtectionCaptchaSimpleCaptchaResponseContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateAttackProtectionCaptchaResponseContent]
            Captcha configuration successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/captcha",
            method="PATCH",
            json={
                "active_provider_id": active_provider_id,
                "arkose": convert_and_respect_annotation_metadata(
                    object_=arkose, annotation=AttackProtectionUpdateCaptchaArkose, direction="write"
                ),
                "auth_challenge": convert_and_respect_annotation_metadata(
                    object_=auth_challenge, annotation=AttackProtectionCaptchaAuthChallengeRequest, direction="write"
                ),
                "hcaptcha": convert_and_respect_annotation_metadata(
                    object_=hcaptcha, annotation=AttackProtectionUpdateCaptchaHcaptcha, direction="write"
                ),
                "friendly_captcha": convert_and_respect_annotation_metadata(
                    object_=friendly_captcha, annotation=AttackProtectionUpdateCaptchaFriendlyCaptcha, direction="write"
                ),
                "recaptcha_enterprise": convert_and_respect_annotation_metadata(
                    object_=recaptcha_enterprise,
                    annotation=AttackProtectionUpdateCaptchaRecaptchaEnterprise,
                    direction="write",
                ),
                "recaptcha_v2": convert_and_respect_annotation_metadata(
                    object_=recaptcha_v_2, annotation=AttackProtectionUpdateCaptchaRecaptchaV2, direction="write"
                ),
                "simple_captcha": simple_captcha,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateAttackProtectionCaptchaResponseContent,
                    parse_obj_as(
                        type_=UpdateAttackProtectionCaptchaResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCaptchaClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetAttackProtectionCaptchaResponseContent]:
        """
        Get the CAPTCHA configuration for your client.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetAttackProtectionCaptchaResponseContent]
            Captcha configuration successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/captcha",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAttackProtectionCaptchaResponseContent,
                    parse_obj_as(
                        type_=GetAttackProtectionCaptchaResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        active_provider_id: typing.Optional[AttackProtectionCaptchaProviderId] = OMIT,
        arkose: typing.Optional[AttackProtectionUpdateCaptchaArkose] = OMIT,
        auth_challenge: typing.Optional[AttackProtectionCaptchaAuthChallengeRequest] = OMIT,
        hcaptcha: typing.Optional[AttackProtectionUpdateCaptchaHcaptcha] = OMIT,
        friendly_captcha: typing.Optional[AttackProtectionUpdateCaptchaFriendlyCaptcha] = OMIT,
        recaptcha_enterprise: typing.Optional[AttackProtectionUpdateCaptchaRecaptchaEnterprise] = OMIT,
        recaptcha_v_2: typing.Optional[AttackProtectionUpdateCaptchaRecaptchaV2] = OMIT,
        simple_captcha: typing.Optional[AttackProtectionCaptchaSimpleCaptchaResponseContent] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateAttackProtectionCaptchaResponseContent]:
        """
        Update existing CAPTCHA configuration for your client.

        Parameters
        ----------
        active_provider_id : typing.Optional[AttackProtectionCaptchaProviderId]

        arkose : typing.Optional[AttackProtectionUpdateCaptchaArkose]

        auth_challenge : typing.Optional[AttackProtectionCaptchaAuthChallengeRequest]

        hcaptcha : typing.Optional[AttackProtectionUpdateCaptchaHcaptcha]

        friendly_captcha : typing.Optional[AttackProtectionUpdateCaptchaFriendlyCaptcha]

        recaptcha_enterprise : typing.Optional[AttackProtectionUpdateCaptchaRecaptchaEnterprise]

        recaptcha_v_2 : typing.Optional[AttackProtectionUpdateCaptchaRecaptchaV2]

        simple_captcha : typing.Optional[AttackProtectionCaptchaSimpleCaptchaResponseContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateAttackProtectionCaptchaResponseContent]
            Captcha configuration successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/captcha",
            method="PATCH",
            json={
                "active_provider_id": active_provider_id,
                "arkose": convert_and_respect_annotation_metadata(
                    object_=arkose, annotation=AttackProtectionUpdateCaptchaArkose, direction="write"
                ),
                "auth_challenge": convert_and_respect_annotation_metadata(
                    object_=auth_challenge, annotation=AttackProtectionCaptchaAuthChallengeRequest, direction="write"
                ),
                "hcaptcha": convert_and_respect_annotation_metadata(
                    object_=hcaptcha, annotation=AttackProtectionUpdateCaptchaHcaptcha, direction="write"
                ),
                "friendly_captcha": convert_and_respect_annotation_metadata(
                    object_=friendly_captcha, annotation=AttackProtectionUpdateCaptchaFriendlyCaptcha, direction="write"
                ),
                "recaptcha_enterprise": convert_and_respect_annotation_metadata(
                    object_=recaptcha_enterprise,
                    annotation=AttackProtectionUpdateCaptchaRecaptchaEnterprise,
                    direction="write",
                ),
                "recaptcha_v2": convert_and_respect_annotation_metadata(
                    object_=recaptcha_v_2, annotation=AttackProtectionUpdateCaptchaRecaptchaV2, direction="write"
                ),
                "simple_captcha": simple_captcha,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateAttackProtectionCaptchaResponseContent,
                    parse_obj_as(
                        type_=UpdateAttackProtectionCaptchaResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_client_id_paypal import ConnectionClientIdPaypal
from .connection_client_secret_paypal import ConnectionClientSecretPaypal
from .connection_freeform_scopes_paypal import ConnectionFreeformScopesPaypal
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_paypal import ConnectionScopePaypal
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum


class ConnectionOptionsPaypal(ConnectionOptionsCommon):
    """
    Options for the 'paypal' and 'paypal-sandbox' connections
    """

    client_id: typing.Optional[ConnectionClientIdPaypal] = None
    client_secret: typing.Optional[ConnectionClientSecretPaypal] = None
    freeform_scopes: typing.Optional[ConnectionFreeformScopesPaypal] = None
    scope: typing.Optional[ConnectionScopePaypal] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    address: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests the 'address' scope from PayPal to access the user's address information.
    """

    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests the 'email' scope from PayPal to access the user's email address.
    """

    phone: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests the 'phone' scope from PayPal to access the user's phone number.
    """

    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests the 'profile' scope from PayPal to access basic profile information including first name, last name, date of birth, time zone, locale, and language. This scope is always enabled by the system.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import encode_path_param
from ....core.pagination import AsyncPager, SyncPager
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....errors.bad_request_error import BadRequestError
from ....errors.conflict_error import ConflictError
from ....errors.forbidden_error import ForbiddenError
from ....errors.not_found_error import NotFoundError
from ....errors.too_many_requests_error import TooManyRequestsError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.list_organization_member_roles_offset_paginated_response_content import (
    ListOrganizationMemberRolesOffsetPaginatedResponseContent,
)
from ....types.role import Role
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRolesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        id: str,
        user_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]:
        """
        Retrieve detailed list of roles assigned to a given user within the context of a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action only returns the roles associated with the specified Organization; any roles assigned to the user within other Organizations are not included.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]
            Roles successfully retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}/members/{encode_path_param(user_id)}/roles",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrganizationMemberRolesOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListOrganizationMemberRolesOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.roles
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    id,
                    user_id,
                    page=page + 1,
                    per_page=per_page,
                    include_totals=include_totals,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def assign(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Assign one or more [roles](https://auth0.com/docs/manage-users/access-control/rbac) to a user to determine their access for a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action assigns roles to a user only for the specified Organization. Roles cannot be assigned to a user across multiple Organizations in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        roles : typing.Sequence[str]
            List of roles IDs to associated with the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}/members/{encode_path_param(user_id)}/roles",
            method="POST",
            json={
                "roles": roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Remove one or more Organization-specific [roles](https://auth0.com/docs/manage-users/access-control/rbac) from a given user.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action removes roles from a user in relation to the specified Organization. Roles assigned to the user within a different Organization cannot be managed in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            User ID of the organization member to remove roles from.

        roles : typing.Sequence[str]
            List of roles IDs associated with the organization member to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}/members/{encode_path_param(user_id)}/roles",
            method="DELETE",
            json={
                "roles": roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRolesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        id: str,
        user_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]:
        """
        Retrieve detailed list of roles assigned to a given user within the context of a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action only returns the roles associated with the specified Organization; any roles assigned to the user within other Organizations are not included.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]
            Roles successfully retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}/members/{encode_path_param(user_id)}/roles",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrganizationMemberRolesOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListOrganizationMemberRolesOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.roles
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        id,
                        user_id,
                        page=page + 1,
                        per_page=per_page,
                        include_totals=include_totals,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def assign(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Assign one or more [roles](https://auth0.com/docs/manage-users/access-control/rbac) to a user to determine their access for a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action assigns roles to a user only for the specified Organization. Roles cannot be assigned to a user across multiple Organizations in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        roles : typing.Sequence[str]
            List of roles IDs to associated with the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}/members/{encode_path_param(user_id)}/roles",
            method="POST",
            json={
                "roles": roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Remove one or more Organization-specific [roles](https://auth0.com/docs/manage-users/access-control/rbac) from a given user.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action removes roles from a user in relation to the specified Organization. Roles assigned to the user within a different Organization cannot be managed in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            User ID of the organization member to remove roles from.

        roles : typing.Sequence[str]
            List of roles IDs associated with the organization member to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}/members/{encode_path_param(user_id)}/roles",
            method="DELETE",
            json={
                "roles": roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .organization_member_effective_role_source import OrganizationMemberEffectiveRoleSource


class OrganizationMemberEffectiveRole(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    Role ID
    """

    name: str = pydantic.Field()
    """
    Role name
    """

    description: str = pydantic.Field()
    """
    Role description
    """

    sources: typing.List[OrganizationMemberEffectiveRoleSource] = pydantic.Field()
    """
    Sources of the role assignment (direct or through group membership)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_enrollment_auth_method_enum import UserEnrollmentAuthMethodEnum
from .user_enrollment_status_enum import UserEnrollmentStatusEnum


class UsersEnrollment(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of this enrollment.
    """

    status: typing.Optional[UserEnrollmentStatusEnum] = None
    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of enrollment.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of enrollment (usually phone number).
    """

    identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Device identifier (usually phone identifier) of this enrollment.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Phone number for this enrollment.
    """

    auth_method: typing.Optional[UserEnrollmentAuthMethodEnum] = None
    enrolled_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Start date and time of this enrollment.
    """

    last_auth: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last authentication date and time of this enrollment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_users_refreshToken_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "users.refresh_token.list_.0"
    client = get_client(test_id)
    client.users.refresh_token.list(
        user_id="user_id",
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/users/user_id/refresh-tokens", {"from": "from", "take": "1"}, 1)


def test_users_refreshToken_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "users.refresh_token.delete.0"
    client = get_client(test_id)
    client.users.refresh_token.delete(
        user_id="user_id",
    )
    verify_request_count(test_id, "DELETE", "/users/user_id/refresh-tokens", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .acul_head_tag_attributes import AculHeadTagAttributes
from .acul_head_tag_content import AculHeadTagContent


class AculHeadTag(UniversalBaseModel):
    tag: typing.Optional[str] = pydantic.Field(default=None)
    """
    Any HTML element valid for use in the head tag
    """

    attributes: typing.Optional[AculHeadTagAttributes] = None
    content: typing.Optional[AculHeadTagContent] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_role_assigned_object_group_0 import (
    EventStreamCloudEventGroupRoleAssignedObjectGroup0,
)
from .event_stream_cloud_event_group_role_assigned_object_group_1 import (
    EventStreamCloudEventGroupRoleAssignedObjectGroup1,
)
from .event_stream_cloud_event_group_role_assigned_object_group_2 import (
    EventStreamCloudEventGroupRoleAssignedObjectGroup2,
)

EventStreamCloudEventGroupRoleAssignedObjectGroup = typing.Union[
    EventStreamCloudEventGroupRoleAssignedObjectGroup0,
    EventStreamCloudEventGroupRoleAssignedObjectGroup1,
    EventStreamCloudEventGroupRoleAssignedObjectGroup2,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .hook_dependencies import HookDependencies


class CreateHookResponseContent(UniversalBaseModel):
    trigger_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="triggerId"),
        pydantic.Field(alias="triggerId", description="Trigger ID"),
    ] = None
    """
    Trigger ID
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of this hook.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of this hook.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this hook will be executed (true) or ignored (false).
    """

    script: typing.Optional[str] = pydantic.Field(default=None)
    """
    Code to be executed when this hook runs.
    """

    dependencies: typing.Optional[HookDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .log_stream_datadog_response_schema import LogStreamDatadogResponseSchema
from .log_stream_event_bridge_response_schema import LogStreamEventBridgeResponseSchema
from .log_stream_event_grid_response_schema import LogStreamEventGridResponseSchema
from .log_stream_http_response_schema import LogStreamHttpResponseSchema
from .log_stream_mixpanel_response_schema import LogStreamMixpanelResponseSchema
from .log_stream_segment_response_schema import LogStreamSegmentResponseSchema
from .log_stream_splunk_response_schema import LogStreamSplunkResponseSchema
from .log_stream_sumo_response_schema import LogStreamSumoResponseSchema

LogStreamResponseSchema = typing.Union[
    LogStreamHttpResponseSchema,
    LogStreamEventBridgeResponseSchema,
    LogStreamEventGridResponseSchema,
    LogStreamDatadogResponseSchema,
    LogStreamSplunkResponseSchema,
    LogStreamSumoResponseSchema,
    LogStreamSegmentResponseSchema,
    LogStreamMixpanelResponseSchema,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_non_persistent_attrs import ConnectionNonPersistentAttrs


class ConnectionOptionsCommon(UniversalBaseModel):
    """
    Common attributes for connection options including non-persistent attributes and Cross App Access
    """

    non_persistent_attrs: typing.Optional[ConnectionNonPersistentAttrs] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
